package service

import (
	"context"

	"git.pbiernat.dev/egommerce/api-entities/model"
	"git.pbiernat.dev/egommerce/go-api-pkg/api"
	"git.pbiernat.dev/egommerce/go-api-pkg/fluentd"
	amqp "git.pbiernat.dev/egommerce/go-api-pkg/rabbitmq"
	"git.pbiernat.dev/egommerce/order-service/internal/app/event"
	"github.com/go-redis/redis/v8"
	"github.com/jackc/pgx/v4/pgxpool"
	base "github.com/streadway/amqp"
)

const (
	SERVICE_USER_AGENT = "order-httpclient"
)

type OrderService struct {
	dbConn *pgxpool.Pool
	redis  *redis.Client
	ebCh   *base.Channel
	log    *fluentd.Logger
}

func NewOrderService(dbConn *pgxpool.Pool, redis *redis.Client, chn *base.Channel, log *fluentd.Logger) *OrderService {
	return &OrderService{dbConn, redis, chn, log}
}

func (s *OrderService) Log(format string, val ...any) {
	s.log.Log(format, val...)
}

// Refactor
func (s *OrderService) CreateOrder(ctx context.Context, basketID string) (*model.OrderModel, error) {
	basketAPI := api.NewBasketAPI(SERVICE_USER_AGENT, s.redis)
	basket, err := basketAPI.GetBasket(basketID)
	if err != nil {
		return nil, err
	}

	order := new(model.OrderModel)
	order.State = basket.State // FIXME: are the same status?

	sql := `INSERT INTO ordering."order"(state) VALUES($1) RETURNING id`
	if err := s.dbConn.QueryRow(ctx, sql, order.State).Scan(&order.ID); err != nil {
		return nil, err
	}

	items, err := basketAPI.GetBasketItems(basket.ID)
	if err != nil {
		return nil, err
	}

	for _, item := range items {
		sql := `INSERT INTO ordering.order_item(order_id,product_id,price,quantity) VALUES($1,$2,$3,$4)`
		if _, err := s.dbConn.Exec(ctx, sql, order.ID, item.ProductID, item.Price, item.Quantity); err != nil {
			return nil, err
		}
	}

	return order, nil
}
func (s *OrderService) UpdateOrderStatus(reqID, orderID, status string) (string, error) {
	s.log.Log("Update order#%s status to %s", orderID, status)

	msg := &event.StatusUpdateEvent{Event: event.NewEvent(reqID), OrderID: orderID, Status: status}
	amqp.Publish(s.ebCh, "api-events", "order.email.statusUpdate", msg)

	return orderID, nil
}
